/**
 * Copyright (c) 2004-2011 QOS.ch
 * All rights reserved.
 *
 * Permission is hereby granted, free  of charge, to any person obtaining
 * a  copy  of this  software  and  associated  documentation files  (the
 * "Software"), to  deal in  the Software without  restriction, including
 * without limitation  the rights to  use, copy, modify,  merge, publish,
 * distribute,  sublicense, and/or sell  copies of  the Software,  and to
 * permit persons to whom the Software  is furnished to do so, subject to
 * the following conditions:
 *
 * The  above  copyright  notice  and  this permission  notice  shall  be
 * included in all copies or substantial portions of the Software.
 *
 * THE  SOFTWARE IS  PROVIDED  "AS  IS", WITHOUT  WARRANTY  OF ANY  KIND,
 * EXPRESS OR  IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE  WARRANTIES OF
 * MERCHANTABILITY,    FITNESS    FOR    A   PARTICULAR    PURPOSE    AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE,  ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 */
package org.slf4j.reload4j.testHarness;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.apache.log4j.AppenderSkeleton;
import org.apache.log4j.spi.LoggingEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class RecursiveAppender extends AppenderSkeleton {

    int diff = new Random().nextInt();
    int activationDelay = 0;
    String loggerName = "org.slf4j.impl.RecursiveAppender" + diff;

    public List<LoggingEvent> events = new ArrayList<>();

    public RecursiveAppender() {
        System.out.println("XXXXXXX entering RecursiveAppender constructor");
        Logger logger = LoggerFactory.getLogger(loggerName);
        logger.info("Calling a logger in the constructor");
        System.out.println("exiting RecursiveAppender constructor");
    }

    protected void append(LoggingEvent e) {
        events.add(e);
    }

    public void close() {
    }

    public boolean requiresLayout() {
        return false;
    }

    @Override
    public void activateOptions() {
        System.out.println("entering RecursiveAppender.activateOptions");
        if (activationDelay > 0) {
            Logger logger = LoggerFactory.getLogger(loggerName);
            logger.info("About to wait {} millis", activationDelay);
            try {
                Thread.sleep(activationDelay);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            logger.info("Done waiting {} millis", activationDelay);
        }
        super.activateOptions();

        System.out.println("exiting RecursiveAppender.activateOptions");
    }

    public int getActivationDelay() {
        return activationDelay;
    }

    public void setActivationDelay(int activationDelay) {
        this.activationDelay = activationDelay;
    }

}
